package com.kakao.sdk.partner.auth

import android.app.Activity
import android.content.Context
import android.content.Intent
import android.net.Uri
import android.os.Bundle
import android.os.Handler
import android.os.Looper
import android.os.ResultReceiver
import com.google.gson.annotations.SerializedName
import com.kakao.sdk.auth.TokenManagerProvider
import com.kakao.sdk.common.KakaoSdk
import com.kakao.sdk.common.model.KakaoSdkError
import com.kakao.sdk.common.util.SdkLog
import com.kakao.sdk.partner.auth.model.AgeAuthError
import com.kakao.sdk.partner.auth.model.AgeAuthErrorCause
import com.kakao.sdk.partner.auth.model.AgeAuthLevel
import com.kakao.sdk.partner.auth.model.Description

class AgeAuthClient {
    fun verifyAgeWithAuthentication (
        context: Context,
        ageLimit : Int? = null,
        authLevel : AgeAuthLevel? = null,
        skipTerms : Boolean? = null,
        adultsOnly : Boolean? = null,
        underAge : Boolean? = false,
        callback: (error: Throwable?) -> Unit) {

        val uri = makeUri(ageLimit, authLevel, skipTerms, adultsOnly, underAge)

        val intent = Intent(context, AgeAuthActivity::class.java)
            .putExtra(com.kakao.sdk.auth.Constants.KEY_BUNDLE, Bundle().apply {
                putParcelable(com.kakao.sdk.auth.Constants.KEY_RESULT_RECEIVER, resultReceiver(callback))
                putParcelable(com.kakao.sdk.auth.Constants.KEY_FULL_URI, uri)
            }).addFlags(Intent.FLAG_ACTIVITY_NEW_TASK) // (signup 시 SDK가 갖고 있던) Application Context 로 Activity를 띄우기 위해 설정

        try {
            context.startActivity(intent)
        } catch (startActivityError: Throwable) {
            SdkLog.e(startActivityError)
            callback(startActivityError)
        }
    }

    @JvmSynthetic
    fun resultReceiver(
        callback: (error: Throwable?) -> Unit
    ): ResultReceiver =
        object: ResultReceiver(Handler(Looper.getMainLooper())) {
            override fun onReceiveResult(resultCode: Int, resultData: Bundle?) {
                SdkLog.d("***** Age Auth Status: $resultData")
                if (resultCode == Activity.RESULT_OK) {
                    SdkLog.i("Activity.RESULT_OK")
                    val uri = resultData?.getParcelable<Uri>(com.kakao.sdk.auth.Constants.KEY_URL)

                    var param = 0
                    try {
                        param = uri?.getQueryParameter(Constants.STATUS)?.toInt() ?: -1
                    } catch (e : Exception) {
                        param = -9999
                        SdkLog.i(e)
                    }

                    if (param == 0) { //성공
                        callback(null)
                        return
                    }

                    val cause = AgeAuthErrorCause.values().find { it.status == param }
                    if (cause == null) {
                        callback(param?.let { AgeAuthError(AgeAuthErrorCause.UNKNOWN.status, "unknown error.") })
                        return
                    }

                    val value = cause.status
                    val description = cause.javaClass.getField(cause.name)?.getAnnotation(
                        Description::class.java)?.value
                    SdkLog.i("status ${value} , description ${description}")

                    callback(description?.let { it -> AgeAuthError(value, it) } )

                } else if (resultCode == Activity.RESULT_CANCELED) {
                    var error = resultData?.getSerializable(com.kakao.sdk.auth.Constants.KEY_EXCEPTION) as KakaoSdkError
                    callback(error)
                } else {
                    var error = IllegalArgumentException("Unknown status in AgeAuthClient#onReceivedResult()")
                    callback(error)
                }
            }
        }

    fun makeUri(
        ageLimit : Int?,
        authLevel : AgeAuthLevel?,
        skipTerms : Boolean?,
        adultsOnly : Boolean?,
        underAge : Boolean? = false) =
        Uri.Builder().scheme(com.kakao.sdk.common.Constants.SCHEME)
            .authority(KakaoSdk.hosts.mobileAccount)
            .path(Constants.AGE_AUTH_PATH)
            .appendQueryParameter(Constants.RETURN_URL, KakaoSdk.applicationContextInfo.customScheme + "://" + Constants.AGE_AUTH_SCHEME)
            .appendQueryParameter(Constants.TOKEN_TYPE , Constants.API_TYPE)
            .appendQueryParameter(Constants.ACCESS_TOKEN , TokenManagerProvider.instance.manager.getToken()?.accessToken)
            .appendQueryParameter(Constants.AUTH_FROM , KakaoSdk.appKey)
            .apply {
                authLevel?.javaClass?.getField(authLevel.name)?.getAnnotation(SerializedName::class.java)?.value?.let {
                    appendQueryParameter(Constants.AUTH_LEVEL, it)
                }
                ageLimit?.let { appendQueryParameter(Constants.AGE_LIMIT, it.toString()) }
                skipTerms?.let { appendQueryParameter(Constants.SKIP_TERM, it.toString()) }
                adultsOnly?.let { appendQueryParameter(Constants.ADULTS_ONLY, it.toString()) }
                underAge?.let { appendQueryParameter(Constants.UNDER_AGE, it.toString()) }
            }.build()


    companion object {
        /**
         * 간편한 API 호출을 위해 기본 제공되는 singleton 객체
         */
        @JvmStatic
        val instance by lazy { AgeAuthClient() }
    }
}