/*
  Copyright 2020 Kakao Corp.

  Licensed under the Apache License, Version 2.0 (the "License");
  you may not use this file except in compliance with the License.
  You may obtain a copy of the License at

     http://www.apache.org/licenses/LICENSE-2.0

  Unless required by applicable law or agreed to in writing, software
  distributed under the License is distributed on an "AS IS" BASIS,
  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  See the License for the specific language governing permissions and
  limitations under the License.
 */
package com.kakao.sdk.partner.user



import android.content.Context
import com.kakao.sdk.auth.network.rxKapiWithOAuth
import com.kakao.sdk.common.util.KakaoJson
import com.kakao.sdk.common.network.ApiFactory
import com.kakao.sdk.common.network.RxOperations
import com.kakao.sdk.common.util.SdkLog
import com.kakao.sdk.partner.user.model.AgeAuthInfo
import com.kakao.sdk.partner.user.model.PartnerUser
import com.kakao.sdk.partner.user.network.RxUserOperations

import com.kakao.sdk.user.RxUserApiClient
import io.reactivex.Completable
import io.reactivex.Single
import io.reactivex.schedulers.Schedulers

// TODO: Java에서 호출 가능하게


/**
 * @suppress
 */
val RxUserApiClient.partnerApi: RxPartnerUserApi
    get() = ApiFactory.rxKapiWithOAuth.create(RxPartnerUserApi::class.java)

/**
 * 파트너용 /v2/user/me API 를 호출. 사용자에 대한 다양한 정보 획득.
 */
@JvmOverloads
fun RxUserApiClient.meForPartner(
    propertyKeys: List<String>? = null,
    secureReSource: Boolean = true
): Single<PartnerUser> =
    partnerApi.me(
        if (propertyKeys == null) null else KakaoJson.toJson(propertyKeys),
        secureReSource
    )
        .compose(RxOperations.handleApiError())
        .compose(authOperations.handleApiError())

/**
 * 앱 연결 상태가 **PREREGISTER** 상태의 사용자에 대하여 앱 연결 요청. **자동연결** 설정을 비활성화한 앱에서 사용.
 */
@JvmOverloads
fun RxUserApiClient.signup(properties: Map<String, String>? = null): Completable =
        partnerApi.signup(properties)
        .compose(RxOperations.handleCompletableError())
        .compose(authOperations.handleCompletableError())
        .compose(RxUserOperations.instance.handleCompletableError())


/**
 * 연령인증이 필요한 시점 연령인증 결과 조회를 확인 하기 위해 사용.
 *
 * @param ageLimit     reponse의 bypass_age_limit를 판단하는 기준 제한 연령
 * @param propertyKeys 추가 동의를 필요로 하는 인증 정보를 응답에 포함하고 싶은 경우, 해당 키 리스트
 * @param callback     연령인증 조회 결과 [AgeAuthInfo] 수신
 *
 */
@JvmOverloads
fun RxUserApiClient.ageAuthInfo(
    ageLimit: Int? = null,
    propertyKeys: List<String>? = null) : Single<AgeAuthInfo> =
    partnerApi.ageAuthInfo(ageLimit, if (propertyKeys == null) null else KakaoJson.toJson(propertyKeys))
    .compose(RxOperations.handleApiError())
    .compose(authOperations.handleApiError())